// PSKReporterTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
//#include <vld.h>
#include "pskreporter.h"
#include <windows.h>

#include <string>
#include <fstream>

#include "utfconverter.h"

void PrintResponse(int rc, const char *fn)
{
	wchar_t resp[256];

	ReporterGetInformation(resp, sizeof(resp) / sizeof(resp[0]));

	if (resp[0] || rc < 0) {
		printf("%s: returned rc=%d and message '%S'\n", fn, rc, resp);
	}
}

std::wstring mangleComma(wchar_t *s)
{
	int l = (int) wcslen(s) + 1;

	wchar_t *p = s;

	while (*p) {
		if (*p == ',') 
			*p = 0;
		p++;
	}

	return std::wstring(s, l);
}

void HandleOptions(int argc, wchar_t **argv)
{
	static std::wstring hostname;
	static std::wstring port;

	static std::wstring localInformation;
	static std::wstring remoteInformation;

	int rc;
	static int tentative = 0;

	static int testFlag = REPORTER_SOURCE_TEST;

	//const char *zt = "foo\0bar\0\0";

	//const wchar_t *tt = UtfConverter::FromUtf8(std::string(zt, 9)).c_str();

	for (int argn = 0; argn < argc; argn++) {
		// skip arguments not starting with '-'
		if (argv[argn][0] != '-') {
			if (argv[argn][0] == '@') {
				std::wifstream src(argv[argn] + 1);

				if (src.is_open()) {
					while (!src.eof()) {
						wchar_t linebuff[1024];

						src.getline(linebuff, sizeof(linebuff) / sizeof(linebuff[0]));

						wchar_t *av[32];
						int ac;

						wchar_t *p = linebuff;

						for (ac = 0; *p && ac < sizeof(av) / sizeof(av[0]) - 1; ) {
							while (*p && iswspace(*p)) {
								p++;
							}

							av[ac++] = p;

							while (*p && *p > ' ') {
								p++;
							}

							if (!*p)
								break;

							*p++ = 0;	// Null terminate the argument
						}

						av[ac] = 0;

						HandleOptions(ac, av);
					}
				} else {
					printf("Failed to open '%S'\n", argv[argn] + 1);
				}
			}
			continue;
		}

		switch (argv[argn][1]) {
			default:
				printf("Unrecognized argument '-%c'\n", argv[argn][1]);
				break;

			case 'h':
				hostname = argv[++argn];
				rc = ReporterInitialize(hostname.c_str(), port.c_str());
				PrintResponse(rc, "ReporterInitialize");
				break;

			case 'p':
				port = argv[++argn];
				rc = ReporterInitialize(hostname.c_str(), port.c_str());
				PrintResponse(rc, "ReporterInitialize");
				break;

			case 't':
				rc = ReporterTickle();
				PrintResponse(rc, "ReporterTickle");
				break;

			case 'd':
				tentative = (argv[++argn][0] == '1') ? REPORTER_SOURCE_TENTATIVE : 0;
				break;

			case 'T':
				testFlag = (argv[++argn][0] == '1') ? REPORTER_SOURCE_TEST : 0;
				break;

			case 's':
				{
					int duration = wcstol(argv[++argn], 0, 0);

					Sleep(duration * 1000);
				}
				break;

			case 'l':
				localInformation = mangleComma(argv[++argn]);
				localInformation += std::wstring(L"\0", 1);
				break;

			case 'r':
				remoteInformation = mangleComma(argv[++argn]);
				remoteInformation += std::wstring(L"\0", 1);

				rc = ReporterSeenCallsign(remoteInformation.c_str(), localInformation.c_str(), REPORTER_SOURCE_AUTOMATIC | testFlag | tentative);
				PrintResponse(rc, "ReporterSeenCallsign");
				break;

			case 'L':
				localInformation = argv[++argn];
				localInformation += std::wstring(L"\0", 1);
				break;

			case 'R':
				remoteInformation = argv[++argn];
				remoteInformation += std::wstring(L"\0", 1);

				rc = ReporterSeenCallsign(remoteInformation.c_str(), localInformation.c_str(), REPORTER_SOURCE_AUTOMATIC | testFlag | tentative);
				PrintResponse(rc, "ReporterSeenCallsign");
				break;

			case 'S':
				{
					REPORTER_STATISTICS stats;

					rc = ReporterGetStatistics(&stats, sizeof(stats));
					PrintResponse(rc, "ReporterGetStatistics");
					printf("Host=%S:%S, conn=%d, Sent=%d, buffered=%d, discarded=%d, last=%d, next=%d, cs=%S\n", 
						stats.hostname, stats.port, stats.connected,
							stats.callsigns_sent, stats.callsigns_buffered, stats.callsigns_discarded,
							stats.last_send_time, stats.next_send_time, stats.last_callsign_queued);
					printf("Sent=%d/%d, total=%d/%d\n",
						stats.bytes_sent, stats.packets_sent,
						stats.bytes_sent_total, stats.packets_sent_total);
				}
		}
	}
}

int _tmain(int argc, _TCHAR* argv[])
{
	if (argc == 1) {
		printf("Usage: pskreportertest options | @file\n");
		printf("The file can contain a list of options\n");
		printf("Options:\n");
		printf("    -h hostname     Set the hostname and re-initialize the library\n");
		printf("    -p port         Set the port and re-initialize the library\n");
		printf("    -l information  Provide local information\n");
		printf("    -r information  Provide remote information and report this callsign\n");
		printf("    -L information  Provide local information (using any seps)\n");
		printf("    -R information  Provide remote information and report this callsign (using any seps)\n");
		printf("    -s seconds	    Sleep this many seconds\n");
		printf("    -S       	    print Statistics\n");
		printf("    -d 0/1    	    Set the dodgy callsign flag (aka TENTATIVE)\n");
		printf("    -t        	    Tickle the library\n\n");
		printf("    -T 0/1          Set the testflag (default)\n");
		printf("The information is in the form keyword,value,keyword,value,...\n");
	} else {
		int rc;

		rc = ReporterInitialize(0, 0);
		PrintResponse(rc, "ReporterInitialize");

		HandleOptions(argc - 1, argv + 1);

		rc = ReporterUninitialize();
		PrintResponse(rc, "ReporterUninitialize");
	}

	return 0;
}

